DROP TABLE IF EXISTS `info_instances`;
CREATE TABLE `info_instances` (
  `id` smallint(5) unsigned NOT NULL AUTO_INCREMENT,
  `hostname` varchar(255) NOT NULL,
  `port` smallint(5) unsigned NOT NULL,
  `server_id` int(10) unsigned NOT NULL,
  `mysql_version` varchar(25) NOT NULL,
  `os` varchar(25) NOT NULL DEFAULT 'Solaris',
  `os_version` varchar(25) NOT NULL DEFAULT '10',
  `architecture` varchar(25) NOT NULL DEFAULT 'x86',
  `bits` tinyint(3) unsigned NOT NULL DEFAULT '64',
  `notes` varchar(255) DEFAULT NULL,
  PRIMARY KEY (`id`)
) ENGINE=MYISAM DEFAULT CHARSET=latin1 ;


DROP TABLE IF EXISTS `info_object_size`;
CREATE TABLE `info_object_size` (
  `instance_id` int(10) unsigned NOT NULL,
  `date_time` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
  `db_name` varchar(64) NOT NULL,
  `tb_name` varchar(64) NOT NULL,
  `engine` varchar(20) NOT NULL,
  `data_size` int(10) unsigned NOT NULL,
  `index_size` int(10) unsigned NOT NULL,
  `data_index_size` int(10) unsigned NOT NULL,
  `data_free` int(10) unsigned NOT NULL,
  `row_count` int(10) unsigned NOT NULL
) ENGINE=MYISAM DEFAULT CHARSET=latin1 ;


DROP TABLE IF EXISTS `info_objects`;
CREATE TABLE `info_objects` (
  `instance_id` int(10) unsigned NOT NULL,
  `db_name` varchar(64) NOT NULL,
  `tb_sp_name` varchar(64) NOT NULL,
  `type` char(1) NOT NULL DEFAULT 't'
) ENGINE=MYISAM DEFAULT CHARSET=latin1 ;


DROP TABLE IF EXISTS `info_privileges`;
CREATE TABLE `info_privileges` (
  `instance_id` tinyint(3) unsigned NOT NULL,
  `user_id` mediumint(8) unsigned NOT NULL,
  `database` varchar(64) NOT NULL,
  `privileges` text NOT NULL
) ENGINE=MYISAM DEFAULT CHARSET=latin1 ;


DROP TABLE IF EXISTS `info_users`;
CREATE TABLE `info_users` (
  `id` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `instance_id` tinyint(3) unsigned NOT NULL,
  `username` varchar(16) NOT NULL,
  `hostname` varchar(60) NOT NULL,
  `password` char(41) CHARACTER SET latin1 COLLATE latin1_bin NOT NULL,
  PRIMARY KEY (`id`),
  KEY `instance_id_idx` (`instance_id`),
  KEY `username_idx` (`username`)
) ENGINE=MYISAM DEFAULT CHARSET=latin1 ;

DROP TABLE IF EXISTS `version`;
CREATE TABLE `version` (
  `ID` int(10) unsigned NOT NULL AUTO_INCREMENT,
  `VERSION` varchar(10) NOT NULL,
  `UPDATED_TIMESTAMP` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP,
  PRIMARY KEY (`ID`)
) ENGINE=MYISAM AUTO_INCREMENT=1 DEFAULT CHARSET=latin1;


DELIMITER $$
CREATE  PROCEDURE `info_object_size_reporting`()
BEGIN

        DECLARE HOST_NAME varchar(60);
        DECLARE PORT_NUM INT;
        DECLARE OS_TYPE varchar(15);
        DECLARE OS_ARCH varchar(5);
        DECLARE MYSQL_VERSION_NUM varchar(10);
        DECLARE SERVER_ID_NUM INT;

        DECLARE MACHINE_EXISTS INT;
        DECLARE INSTANCE_ID TINYINT;


        SET HOST_NAME=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='hostname');
        SET PORT_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='PORT');
        SET OS_TYPE=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_OS');
        SET OS_ARCH=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_MACHINE');
        SET MYSQL_VERSION_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION');
        SET SERVER_ID_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='SERVER_ID');



        SET MACHINE_EXISTS = (
            SELECT COUNT(*)
            FROM crib.info_instances
            WHERE `hostname`=HOST_NAME and
            `port`=PORT_NUM
        );
        IF MACHINE_EXISTS > 0  THEN

            UPDATE crib.info_instances SET server_id=SERVER_ID_NUM, mysql_version=MYSQL_VERSION_NUM, os=OS_TYPE, architecture=OS_ARCH
            WHERE hostname=HOST_NAME and
            port=PORT_NUM;

        ELSE

            INSERT into crib.info_instances (`hostname`,`port`,`server_id`,`mysql_version`,`os`,`architecture`)
            values (HOST_NAME,PORT_NUM,SERVER_ID_NUM,MYSQL_VERSION_NUM,OS_TYPE,OS_ARCH);

        END IF;

        SET INSTANCE_ID=(SELECT id from crib.info_instances WHERE `hostname`=HOST_NAME and `port`=PORT_NUM order by 1 asc limit 1);



                INSERT INTO info_object_size
                        SELECT
                                INSTANCE_ID,
                                current_timestamp(),
                                table_schema,
                                table_name,
                                engine,
                                ROUND(data_length/1024/1024,0),
                                ROUND(index_length/1024/1024,0),
                                ROUND(data_length/1024/1024,0)+ROUND(index_length/1024/1024,0),
                                ROUND(data_free/1024/1024,0),
                                table_rows
                        FROM information_schema.tables
                                WHERE
                                        table_schema != 'information_schema' and
                                        table_schema != 'mysql' and
                                        table_schema != 'securich' and
                                        table_schema != 'crib' and
                                        table_schema not like '%test%';

END $$
DELIMITER ;


DELIMITER $$
CREATE  PROCEDURE `info_reporting`()
BEGIN

        DECLARE HOST_NAME varchar(60);
        DECLARE PORT_NUM INT;
        DECLARE OS_TYPE varchar(15);
        DECLARE OS_ARCH varchar(5);
        DECLARE MYSQL_VERSION_NUM varchar(10);
        DECLARE SERVER_ID_NUM INT;
        DECLARE dbcount int;

        DECLARE MACHINE_EXISTS INT;
        DECLARE INSTANCEID TINYINT;
        DECLARE USER_ID MEDIUMINT;

        DECLARE usernamein varchar(16);
        DECLARE hostnamein varchar(60);
        DECLARE usernameathostname varchar(85);
        DECLARE grants_list long;

        DECLARE done INT DEFAULT 0;

        DECLARE cur_priv CURSOR FOR
            select  username,hostname
            from crib.info_users
                WHERE instance_id=INSTANCEID;

        DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = 1;



        SET HOST_NAME=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='hostname');
        SET PORT_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='PORT');
        SET OS_TYPE=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_OS');
        SET OS_ARCH=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_MACHINE');
        SET MYSQL_VERSION_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION');
        SET SERVER_ID_NUM=(select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='SERVER_ID');



        SET MACHINE_EXISTS = (
            SELECT COUNT(*)
            FROM crib.info_instances
            WHERE `hostname`=HOST_NAME and
            `port`=PORT_NUM
        );
        IF MACHINE_EXISTS > 0  THEN

            UPDATE crib.info_instances SET server_id=SERVER_ID_NUM, mysql_version=MYSQL_VERSION_NUM, os=OS_TYPE, architecture=OS_ARCH
            WHERE hostname=HOST_NAME and
            port=PORT_NUM;

        ELSE

            INSERT into crib.info_instances (`hostname`,`port`,`server_id`,`mysql_version`,`os`,`architecture`)
            values (HOST_NAME,PORT_NUM,SERVER_ID_NUM,MYSQL_VERSION_NUM,OS_TYPE,OS_ARCH);

        END IF;

        SET INSTANCEID=(SELECT id from crib.info_instances WHERE `hostname`=HOST_NAME and `port`=PORT_NUM order by 1 asc limit 1);



        drop temporary table if exists object_list;

        CREATE TEMPORARY TABLE object_list (
            `instance_id` int(10) unsigned NOT NULL,
            `db_name` varchar(64) NOT NULL,
            `tb_sp_name` varchar(64) NOT NULL,
            `type` char(1) NOT NULL
        ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

        insert into object_list
            select INSTANCEID,
            tb.TABLE_SCHEMA,
            tb.TABLE_NAME,
            "t"
            from information_schema.tables tb
            where tb.TABLE_SCHEMA !='information_schema';

        insert into object_list
            select INSTANCEID,
            rt.ROUTINE_SCHEMA,
            rt.ROUTINE_NAME,
            "s"
            from information_schema.routines rt
            where rt.ROUTINE_SCHEMA !='information_schema';


        delete from info_objects
            where not exists (
                select *
                from object_list
                    where info_objects.instance_id=object_list.instance_id and
                    info_objects.db_name=object_list.db_name and
                    info_objects.tb_sp_name=object_list.tb_sp_name and
                    info_objects.type=object_list.type)
            and instance_id=INSTANCEID;

        insert into info_objects
            select *
            from object_list
                where not exists (
                    select *
                    from info_objects
                    where info_objects.instance_id=object_list.instance_id and
                    info_objects.db_name=object_list.db_name and
                    info_objects.tb_sp_name=object_list.tb_sp_name and
                    info_objects.type=object_list.type);




        drop temporary table if exists crib.users_list;

        CREATE TEMPORARY TABLE crib.users_list (
            `instance_id` int(10) unsigned NOT NULL,
            `username` varchar(16)  NOT NULL,
            `hostname` varchar(60) NOT NULL,
            `password` char(41) CHARACTER SET latin1 NOT NULL
        ) ENGINE=InnoDB DEFAULT CHARSET=latin1 ;

        insert into crib.users_list
            select INSTANCEID,
            USER,
            HOST,
            PASSWORD
            from mysql.user;

        delete from crib.info_users
            where not exists (
                select *
                from crib.users_list
                    where info_users.instance_id=users_list.instance_id and
                    info_users.username=users_list.username and
                    info_users.hostname=users_list.hostname and
                    info_users.`password`=users_list.`password`)
            and instance_id=INSTANCEID;

        insert into crib.info_users (instance_id,username,hostname,`password`)
            select *
            from crib.users_list
                where not exists (
                    select *
                    from crib.info_users
                    where info_users.instance_id=users_list.instance_id and
                    info_users.username=users_list.username and
                    info_users.hostname=users_list.hostname and
                    info_users.`password`=users_list.`password`);

        drop table if exists tmp_privileges;

        CREATE TEMPORARY TABLE `tmp_privileges` (
            `grantee` varchar(80) NOT NULL,
            `table_schema` varchar(64) NOT NULL,
            `privileges` text
        ) ENGINE=MyISAM DEFAULT CHARSET=latin1;

        insert into tmp_privileges
            select grantee, "ALL" as table_schema, group_concat(privilege_type order by privilege_type) as privileges
            from information_schema.user_privileges
                group by grantee
            union all
            select grantee, TABLE_SCHEMA as table_schema, group_concat(privilege_type order by privilege_type) as privileges
            from information_schema.schema_privileges
            group by table_schema, grantee;

        delete from info_privileges where instance_id=INSTANCEID;

        OPEN cur_priv;

            cur_priv_loop:WHILE(done=0) DO

                FETCH cur_priv INTO usernamein, hostnamein;

                IF done=1 THEN
                   LEAVE cur_priv_loop;
                END IF;

                SET usernameathostname= concat("'" , usernamein , "'@'" , hostnamein , "'");

                SET USER_ID=(SELECT id from crib.info_users WHERE `username`=usernamein and `hostname`=hostnamein and instance_id=INSTANCEID);

                SET dbcount = (
                    select count(*)
                    from tmp_privileges
                        where grantee=usernameathostname
                    );

                dbcount_loop:WHILE(dbcount > 0) DO

                    SET @db=(
                        select table_schema
                        from tmp_privileges
                            where grantee=usernameathostname
                            limit 1
                        );

                    SET @privs=(select `privileges` from tmp_privileges where grantee=usernameathostname and table_schema=@db);

                    insert into crib.info_privileges values (INSTANCEID,USER_ID,@db,@privs);

                    DELETE FROM tmp_privileges where grantee=usernameathostname and table_schema=@db ;

                    SET dbcount = (
                        select count(*)
                        from tmp_privileges
                            where grantee=usernameathostname
                        );

                END WHILE dbcount_loop;

            END WHILE cur_priv_loop;

        CLOSE cur_priv;
END $$
DELIMITER ;
