#!/bin/bash

###########################
##                       ##
##  CRIB install script  ##
##                       ##
###########################

## Early termination of script through traps taken care of by rollbacking

ARGUMENT=$1

if [ -z "$1" ]
  then
    ARGUMENT=notsilent
fi

clear

if [ "$ARGUMENT" != "silent" ]
then

echo ""
echo ""                                      
echo "                    ,--,     ,---,    "
echo "           __  ,-.,--.'|   ,---.'|    "
echo "         ,' ,'/ /||  |,    |   | :    "
echo "   ,---. '  | |' |\`--'_    :   : :   "
echo "  /     \|  |   ,',' ,'|   :     |,-. "
echo " /    / ''  :  /  '  | |   |   : '  | "
echo ".    ' / |  | '   |  | :   |   |  / : "
echo "'   ; :__;  : |   '  : |__ '   : |: | "
echo "'   | '.'|  , ;   |  | '.'||   | '/ : "
echo "|   :    :---'    ;  :    ;|   :    | "
echo " \   \  /         |  ,   / /    \  /  "
echo "  \`----'           ---\`-'  \`-'----' "
echo "                                                             "
echo " brought to you by Darren Cassar "
echo " http://www.mysqlpreacher.com "
echo ""
echo ""
echo "Anytime you need to cancel installation just press ( Ctrl + C )"
echo ""
echo ""
fi 


## Checking if mysql binary is available
 
 MYSQLBIN=`which mysql 2> /dev/null`
 if [ "$MYSQLBIN" = "" -o ! -x "$MYSQLBIN" ]
 then
   echo "It seems this install script can't find MYSQL binary."
   echo "Please make sure it is in your system by running 'mysql --version'"
   exit 1
 fi
  
 MYSQLDUMPBIN=`which mysqldump 2> /dev/null`
 if [ "$MYSQLDUMPBIN" = "" -o ! -x "$MYSQLDUMPBIN" ]
 then
   echo "It seems this install script can't find MYSQLDUMP binary."
   echo "Please make sure it is in your system by running 'mysqldump --version'"
   exit 1
 fi

## Version control - suggested

 LV=`tail -1 version`                                                  ## LV = Latest Version
 
## Create logs folder to keep logs of the installation.

 if [ ! -d logs ]
  then
   mkdir logs
 fi

## Choose if installation should get the file from the net or just install it from a  local file.

 echo ""
 echo "Would you like to setup source and destination or just source?"
 echo "1. Client and Server "
 echo "2. New Client "
 echo -n "Enter choice (default 2): "
 read -e TOI                                                                  ## TOI = Type of Installation
 
 if [ "$TOI" == "" ]
  then
    TOI=2
 fi
 
 while [ "$TOI" -lt "1" ] && [ "$TOI" -gt "2" ]
 do
    echo -n "Wrong value, please re-enter choice (default 2 i.e. Setup New Source): "
     read -e TOI
      if [ "$TOI" == "" ]
       then
        TOI=2
      fi
 done

 if [ "$TOI" == "1" ]
  then
    echo "Installation starting"

 
## Choose from fresh installation or upgrade
FOU=1

# echo ""
# echo "Do you wish to:"
# echo "1. Do a fresh install "
# echo "2. Upgrade from a previous version "
# echo -n "Enter choice (default 1): "
# read -e FOU                                                           ## FOU = Fresh install or Upgrade

#  if [ "$FOU" == "" ]
#   then
#    FOU=1
#  fi
#
# while [ "$FOU" -lt "1" ] && [ "$FOU" -gt "2" ]
# do
#    echo -n "Wrong value, please re-enter choice (default 1 i.e. fresh installation): "
#     read -e FOU
#      if [ "$FOU" == "" ]
#       then
#        FOU=1
#      fi
# done
 
 
 echo ""
 echo ""
 echo "This section you'll supply the destination (CentRal Information Base) details"
 echo ""

 echo -n "Enter CRIB user name (default root): "
 read -e CRIBUSER                                                           ## CRIBUSER = Source User Name
 if [ "$CRIBUSER" == "" ]
  then
   CRIBUSER=root
 fi
 
## Enter password (masked for security)

 echo -n "Enter CRIB password (default 'toor'): "

 stty_orig=`stty -g`
 trap "echo ''; echo 'Installation aborted during password entry'; stty $stty_orig ; exit" 1 2 15
 stty -echo
 read -e CRIBPASS
 stty $stty_orig
 trap "" 1 2 15

 if [ "$CRIBPASS" == "" ]
  then
   CRIBPASS=toor
 fi

 echo ""
 echo -n "Enter CRIB hostname (default localhost): "
 read -e CRIBHOST                                                           ## CRIBHOST = Source User Name
 if [ "$CRIBHOST" == "" ]
  then
   CRIBHOST=127.0.0.1
 fi
         
 echo -n "Enter CRIB port (default 3306): "
 read -e CRIBPORT                                                           ## CRIBPORT = Source User Name
 if [ "$CRIBPORT" == "" ]
  then
   CRIBPORT=3306
 fi
 

 echo ""
 echo ""
 echo "This section you'll supply the client details"
 echo ""

 echo -n "Enter client user name (default root): "
 read -e SRCUSER                                                           ## SRCUSER = Source User Name
 if [ "$SRCUSER" == "" ]
  then
   SRCUSER=root
 fi 
## Enter password (masked for security)

 echo -n "Enter client password (default 'toor'): "

 stty_orig=`stty -g`
 trap "echo ''; echo 'Installation aborted during password entry'; stty $stty_orig ; exit" 1 2 15
 stty -echo
 read -e SRCPASS
 stty $stty_orig
 trap "" 1 2 15

 if [ "$CRIBPASS" == "" ]
  then
   CRIBPASS=toor
 fi

 echo ""
 echo -n "Enter client hostname (default localhost): "
 read -e SRCHOST                                                           ## SRCHOST = Source User Name
 if [ "$SRCHOST" == "" ]
  then
   SRCHOST=127.0.0.1
 fi
         
 echo -n "Enter client port (default 3306): "
 read -e SRCPORT                                                           ## SRCPORT = Source User Name
 if [ "$SRCPORT" == "" ]
  then
   SRCPORT=3306
 fi


 if [ "$FOU" == 1 ]
 then

    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT --execute="drop database if exists crib"
    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT --execute="create database crib"
    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT crib < crib.sql
    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT crib < crib_reporting.sql
    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT crib < info_user_privileges.sql
    mysql -u $CRIBUSER --password=$CRIBPASS -h $CRIBHOST -P $CRIBPORT crib --execute="insert into version (version) values('$LV');"
    
    mysql -u $SRCUSER --password=$SRCPASS -h $SRCHOST -P $SRCPORT <<EOFMYSQL
    
    drop database if exists crib_client; 
    create database crib_client;

    use crib_client;

    CREATE TABLE crib_client.object_list (
        instance_id int(10) unsigned NOT NULL,
        db_name varchar(64) NOT NULL,
        tb_sp_name varchar(64) NOT NULL,
        type char(1) NOT NULL
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE crib_client.users_list (
        instance_id int(10) unsigned NOT NULL,
        username varchar(16)  NOT NULL,
        hostname varchar(60) NOT NULL,
        password char(41) CHARACTER SET latin1 NOT NULL
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1 ;

    CREATE TABLE crib_client.user_privilege_list (
        grantee varchar(80) NOT NULL,
        table_schema varchar(64) NOT NULL,
        privileges text
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;

EOFMYSQL

    echo "                                                                                "  > ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Source Variables Section                                                      " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_USER=$SRCUSER                                                               " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_PASSWORD=$SRCPASS                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_HOSTNAME=$SRCHOST                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_PORT=$SRCPORT                                                               " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Destination Variables Section                                                 " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_USER=$CRIBUSER                                                              " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_PASSWORD=$CRIBPASS                                                          " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_HOSTNAME=$CRIBHOST                                                          " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_PORT=$CRIBPORT                                                              " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Variables Declaring Section                                                   " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "LOGFILE=/tmp/crib.log                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "LOGERR=/tmp/crib.err                                                            " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf       
    echo "RECIPIENT=myname@domain.com                                                     " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
 fi
 
 elif [ "$TOI" == "2" ]
  then
    echo "New Client starting"
 
## Choose from fresh installation or upgrade
 echo ""
 echo ""
 echo "This section you'll supply the destination (CentRal Information Base) details"
 echo ""

 echo -n "Enter CRIB user name (default root): "
 read -e CRIBUSER                                                           ## CRIBUSER = Source User Name
 if [ "$CRIBUSER" == "" ]
  then
   CRIBUSER=root
 fi
 
## Enter password (masked for security)

 echo -n "Enter CRIB password (default ''): "

 stty_orig=`stty -g`
 trap "echo ''; echo 'Installation aborted during password entry'; stty $stty_orig ; exit" 1 2 15
 stty -echo
 read -e CRIBPASS
 stty $stty_orig
 trap "" 1 2 15

 echo ""
 echo -n "Enter CRIB hostname (default localhost): "
 read -e CRIBHOST                                                           ## CRIBHOST = Source User Name
 if [ "$CRIBHOST" == "" ]
  then
   CRIBHOST=127.0.0.1
 fi
         
 echo -n "Enter CRIB port (default 3306): "
 read -e CRIBPORT                                                           ## CRIBPORT = Source User Name
 if [ "$CRIBPORT" == "" ]
  then
   CRIBPORT=3306
 fi
  
 echo ""
 echo "This section you'll supply the client details"
 echo ""

 echo -n "Enter client user name (default root): "
 read -e SRCUSER                                                           ## SRCUSER = Source User Name
 if [ "$SRCUSER" == "" ]
  then
   SRCUSER=root
 fi 
## Enter password (masked for security)

 echo -n "Enter client password (default ''): "

 stty_orig=`stty -g`
 trap "echo ''; echo 'Installation aborted during password entry'; stty $stty_orig ; exit" 1 2 15
 stty -echo
 read -e SRCPASS
 stty $stty_orig
 trap "" 1 2 15

 echo ""
 echo -n "Enter client hostname (default localhost): "
 read -e SRCHOST                                                           ## SRCHOST = Source User Name
 if [ "$SRCHOST" == "" ]
  then
   SRCHOST=127.0.0.1
 fi
         
 echo -n "Enter client port (default 3306): "
 read -e SRCPORT                                                           ## SRCPORT = Source User Name
 if [ "$SRCPORT" == "" ]
  then
   SRCPORT=3306
 fi



    mysql -u $SRCUSER --password=$SRCPASS -h $SRCHOST -P $SRCPORT <<EOFMYSQL
    
    drop database if exists crib_client; 
    create database crib_client;

    use crib_client;

    CREATE TABLE crib_client.object_list (
        instance_id int(10) unsigned NOT NULL,
        db_name varchar(64) NOT NULL,
        tb_sp_name varchar(64) NOT NULL,
        type char(1) NOT NULL
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1;

    CREATE TABLE crib_client.users_list (
        instance_id int(10) unsigned NOT NULL,
        username varchar(16)  NOT NULL,
        hostname varchar(60) NOT NULL,
        password char(41) CHARACTER SET latin1 NOT NULL
    ) ENGINE=InnoDB DEFAULT CHARSET=latin1 ;

    CREATE TABLE crib_client.user_privilege_list (
        grantee varchar(80) NOT NULL,
        table_schema varchar(64) NOT NULL,
        privileges text
    ) ENGINE=MyISAM DEFAULT CHARSET=latin1;

EOFMYSQL

    echo "                                                                                "  > ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Source Variables Section                                                      " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_USER=$SRCUSER                                                               " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_PASSWORD=$SRCPASS                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_HOSTNAME=$SRCHOST                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "SRC_PORT=$SRCPORT                                                               " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Destination Variables Section                                                 " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_USER=$CRIBUSER                                                              " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_PASSWORD=$CRIBPASS                                                          " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_HOSTNAME=$CRIBHOST                                                          " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "DST_PORT=$CRIBPORT                                                              " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "# Variables Declaring Section                                                   " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "#=============================================================================  " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "LOGFILE=/tmp/crib.log                                                           " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "LOGERR=/tmp/crib.err                                                            " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf       
    echo "RECIPIENT=myname@domain.com                                                     " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
    echo "                                                                                " >> ../CONFIG/crib_$SRCHOST_$SRCPORT.cnf
 
 fi

 echo "Installation complete"
