#!/bin/bash

###################################################################################
##                                                                               ##
##  crib populate to be scheduled every 6-24 hrs for details about the instance ##
##  on row count, data size, etc to be updated in the Central Information Base   ##
##  v 1.0                                                                        ##
##  10th October 2010                                                            ##
##                                                                               ##
###################################################################################


i=0

CONFFILE=$@

if [ "$CONFFILE" = "" ]
  then
     if [ -e my_bck.cnf ]
        then
            CONFFILE=my_bck.cnf
         else
            echo "Please supply at least one configuration file to use"
            exit 1
     fi
fi

SRC_USER=`grep -s -w SRC_USER $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_USER" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_USER
fi

SRC_PASSWORD=`grep -s -w SRC_PASSWORD $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_PASSWORD" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_PASSWORD
fi

SRC_HOSTNAME=`grep -s -w SRC_HOSTNAME $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_HOSTNAME" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_HOSTNAME
fi

SRC_PORT=`grep -s -w SRC_PORT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_PORT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_PORT
fi

DST_USER=`grep -s -w DST_USER $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_USER" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_USER
fi

DST_PASSWORD=( `grep -s -w DST_PASSWORD $CONFFILE | head -1 | cut -d "=" -f 2` )

if [ "$DST_PASSWORD" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_PASSWORD
fi

DST_HOSTNAME=`grep -s -w DST_HOSTNAME $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_HOSTNAME" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_HOSTNAME
fi

DST_PORT=`grep -s -w DST_PORT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_PORT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_PORT
fi

LOGFILE=`grep -s -w LOGFILE $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$LOGFILE" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=LOGFILE
fi

LOGERR=`grep -s -w LOGERR $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$LOGERR" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=LOGERR
fi

RECIPIENT=`grep -s -w RECIPIENT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$RECIPIENT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=RECIPIENT
fi


#=============================================================================
#
# IMP - DO NOT CHANGE UNLESS YOU KNOW WHAT YOU ARE DOING
#
#=============================================================================

voidparamlistlength=${#voidparamlist[*]}

if [ $i != 0 ]
  then
  echo "The following list of parameters need to be included in your my_bck.cnf:"
  echo ""
  for ((i=1;i<=$voidparamlistlength;i++))
  do
    echo "${voidparamlist[$i]} is needed as a parameter in my_bck.cnf"
  done
  echo ""
  echo ""
  echo "Please consult the README file and compile a correct configuration file."
  exit 1
fi


#=============================================================================
#
# Logging - Redirect IO to LOGERR and LOGFILE
#
#=============================================================================

if [ ! -e "$LOGFILE" ]          # Check if LOGFILE exists.
   then
   touch $LOGFILE
fi

if [ ! -e "$LOGERR" ]           # Check if LOGFERR exists.
   then
   touch $LOGERR
fi

#exec 6>&1           # Link file descriptor #6 with stdout.
                    # Saves stdout.
#exec >> $LOGFILE    # stdout replaced with file $LOGFILE.

#exec 7>&2           # Link file descriptor #7 with stderr.
                    # Saves stderr.
#exec 2> $LOGERR     # stderr replaced with file $LOGERR.


##############################################################################


HOST_NAME=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='hostname'" | tail -1`
PORT_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='PORT'"| tail -1`
OS_TYPE=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_OS'"| tail -1`
OS_ARCH=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_MACHINE'"| tail -1`
MYSQL_VERSION_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION'"| tail -1`
SERVER_ID_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='SERVER_ID'"| tail -1`

MACHINE_EXISTS=`mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -s -B -e "SELECT COUNT(*) FROM crib.info_instances WHERE hostname='$HOST_NAME' and port=$PORT_NUM"| tail -1`

if [ "$MACHINE_EXISTS" -gt "0" ] 
then
    mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -e "UPDATE crib.info_instances SET server_id=$SERVER_ID_NUM, mysql_version='$MYSQL_VERSION_NUM', os='$OS_TYPE', architecture='$OS_ARCH' WHERE hostname='$HOST_NAME' and port=$PORT_NUM"
else
    mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -e "INSERT into crib.info_instances (hostname,port,server_id,mysql_version,os,architecture) values ('$HOST_NAME',$PORT_NUM,$SERVER_ID_NUM,'$MYSQL_VERSION_NUM','$OS_TYPE','$OS_ARCH')"
fi

INSTANCE_ID=`mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -s -B -e "select id from crib.info_instances WHERE hostname='$HOST_NAME' and port='$PORT_NUM' order by 1 asc limit 1"`

#Avoiding the script overwriting a dump file with a coincidental run of another script 
rand=$RANDOM$RANDOM$RANDOM$RANDOM$RANDOM$RANDOM

#mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -e "SELECT $INSTANCE_ID, current_timestamp(), table_schema, table_name, engine, ROUND(data_length/1024/1024,0), ROUND(index_length/1024/1024,0), ROUND(data_length/1024/1024,0)+ROUND(index_length/1024/1024,0), ROUND(data_free/1024/1024,0), table_rows FROM information_schema.tables WHERE table_schema != 'information_schema' and table_schema != 'mysql' and table_schema != 'securich' and table_schema != 'crib' and table_schema not like '%test%' into outfile '/tmp/$rand/info_object_size.txt'"
#mysqlimport -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib /tmp/$rand/info_object_size.txt
        
mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT crib_client <<EOFMYSQL
        
        truncate crib_client.object_list;
        truncate crib_client.users_list;
        truncate crib_client.user_privilege_list;
        
        insert into crib_client.object_list
            select $INSTANCE_ID,
            tb.TABLE_SCHEMA,                                                                                                              
            tb.TABLE_NAME,
            "t"
            from information_schema.tables tb
            where tb.TABLE_SCHEMA !='information_schema';

        insert into crib_client.object_list
            select $INSTANCE_ID,
            rt.ROUTINE_SCHEMA,
            rt.ROUTINE_NAME,
            "s"
            from information_schema.routines rt
            where rt.ROUTINE_SCHEMA !='information_schema';
            
        insert into crib_client.users_list
            select $INSTANCE_ID,
            USER,
            HOST,
            PASSWORD
            from mysql.user;
            
        insert into user_privilege_list
            select grantee, "ALL" as table_schema, group_concat(privilege_type order by privilege_type) as privileges
            from information_schema.user_privileges
                group by grantee
            union all
            select grantee, TABLE_SCHEMA as table_schema, group_concat(privilege_type order by privilege_type) as privileges
            from information_schema.schema_privileges
            group by table_schema, grantee;

        rename table object_list to object_list_$rand, user_privilege_list to  user_privilege_list_$rand, users_list to users_list_$rand;
        
EOFMYSQL

mysqldump -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT --opt crib_client |  mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib

mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT crib_client -e "rename table object_list_$rand to object_list , user_privilege_list_$rand to user_privilege_list, users_list_$rand to users_list"        

mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib <<EOFMYSQL

        delete from crib.info_objects
            where not exists (
                select *
                from object_list_$rand
                    where info_objects.instance_id=object_list_$rand.instance_id and
                    info_objects.db_name=object_list_$rand.db_name and
                    info_objects.tb_sp_name=object_list_$rand.tb_sp_name and
                    info_objects.type=object_list_$rand.type)
            and instance_id=$INSTANCE_ID;  

        insert into crib.info_objects
            select *
            from crib.object_list_$rand
                where not exists (
                    select *
                    from crib.info_objects
                    where info_objects.instance_id=object_list_$rand.instance_id and
                    info_objects.db_name=object_list_$rand.db_name and
                    info_objects.tb_sp_name=object_list_$rand.tb_sp_name and
                    info_objects.type=object_list_$rand.type);

        delete from crib.info_users
            where not exists (
                select *
                from crib.users_list_$rand
                    where info_users.instance_id=users_list_$rand.instance_id and
                    info_users.username=users_list_$rand.username and
                    info_users.hostname=users_list_$rand.hostname and
                    info_users.password=users_list_$rand.password)
            and instance_id=$INSTANCE_ID;

        insert into crib.info_users (instance_id,username,hostname,password)
            select *
            from crib.users_list_$rand
                where not exists (
                    select *
                    from crib.info_users
                    where info_users.instance_id=users_list_$rand.instance_id and
                    info_users.username=users_list_$rand.username and
                    info_users.hostname=users_list_$rand.hostname and
                    info_users.password=users_list_$rand.password); 

        drop table if exists user_privilege_list;

        CREATE TEMPORARY TABLE user_privilege_list (
            grantee varchar(80) NOT NULL,
            table_schema varchar(64) NOT NULL,
            privileges text
        ) ENGINE=MyISAM DEFAULT CHARSET=latin1;
        
        insert into user_privilege_list select * from user_privilege_list_$rand;

        delete from info_privileges where instance_id=$INSTANCE_ID;

        call info_user_privileges($INSTANCE_ID); 
        
        drop table if exists users_list_$rand;
        drop table if exists if exists object_list_$rand;
        drop table if exists user_privilege_list_$rand;
EOFMYSQL
