#!/bin/bash

###################################################################################
##                                                                               ##
##  crib table size to be scheduled weekly for records on row count, data size  ##
##  index sizes, data free, etc.                                                 ##
##  v 1.0                                                                        ##
##  10th October 2010                                                            ##
##                                                                               ##
###################################################################################


i=0

CONFFILE=$@

if [ "$CONFFILE" = "" ]
  then
     if [ -e my_bck.cnf ]
        then
            CONFFILE=my_bck.cnf
         else
            echo "Please supply at least one configuration file to use"
            exit 1
     fi
fi

SRC_USER=`grep -s -w SRC_USER $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_USER" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_USER
fi

SRC_PASSWORD=`grep -s -w SRC_PASSWORD $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_PASSWORD" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_PASSWORD
fi

SRC_HOSTNAME=`grep -s -w SRC_HOSTNAME $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_HOSTNAME" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_HOSTNAME
fi

SRC_PORT=`grep -s -w SRC_PORT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$SRC_PORT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=SRC_PORT
fi

DST_USER=`grep -s -w DST_USER $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_USER" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_USER
fi

DST_PASSWORD=( `grep -s -w DST_PASSWORD $CONFFILE | head -1 | cut -d "=" -f 2` )

if [ "$DST_PASSWORD" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_PASSWORD
fi

DST_HOSTNAME=`grep -s -w DST_HOSTNAME $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_HOSTNAME" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_HOSTNAME
fi

DST_PORT=`grep -s -w DST_PORT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$DST_PORT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=DST_PORT
fi

LOGFILE=`grep -s -w LOGFILE $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$LOGFILE" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=LOGFILE
fi

LOGERR=`grep -s -w LOGERR $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$LOGERR" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=LOGERR
fi

RECIPIENT=`grep -s -w RECIPIENT $CONFFILE | head -1 | cut -d "=" -f 2`

if [ "$RECIPIENT" = "" ]
 then
 i=`expr $i + 1`
 voidparamlist[$i]=RECIPIENT
fi


#=============================================================================
#
# IMP - DO NOT CHANGE UNLESS YOU KNOW WHAT YOU ARE DOING
#
#=============================================================================

voidparamlistlength=${#voidparamlist[*]}

if [ $i != 0 ]
  then
  echo "The following list of parameters need to be included in your my_bck.cnf:"
  echo ""
  for ((i=1;i<=$voidparamlistlength;i++))
  do
    echo "${voidparamlist[$i]} is needed as a parameter in my_bck.cnf"
  done
  echo ""
  echo ""
  echo "Please consult the README file and compile a correct configuration file."
  exit 1
fi


#=============================================================================
#
# Logging - Redirect IO to LOGERR and LOGFILE
#
#=============================================================================

if [ ! -e "$LOGFILE" ]          # Check if LOGFILE exists.
   then
   touch $LOGFILE
fi

if [ ! -e "$LOGERR" ]           # Check if LOGFERR exists.
   then
   touch $LOGERR
fi

#exec 6>&1           # Link file descriptor #6 with stdout.
                    # Saves stdout.
#exec >> $LOGFILE    # stdout replaced with file $LOGFILE.

#exec 7>&2           # Link file descriptor #7 with stderr.
                    # Saves stderr.
#exec 2> $LOGERR     # stderr replaced with file $LOGERR.


##############################################################################


HOST_NAME=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='hostname'" | tail -1`
PORT_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='PORT'"| tail -1`
OS_TYPE=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_OS'"| tail -1`
OS_ARCH=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION_COMPILE_MACHINE'"| tail -1`
MYSQL_VERSION_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='VERSION'"| tail -1`
SERVER_ID_NUM=`mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT -s -B -e "select VARIABLE_VALUE from information_schema.global_variables where VARIABLE_NAME='SERVER_ID'"| tail -1`

MACHINE_EXISTS=`mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -s -B -e "SELECT COUNT(*) FROM crib.info_instances WHERE hostname='$HOST_NAME' and port=$PORT_NUM"| tail -1`

if [ "$MACHINE_EXISTS" -gt "0" ] 
then
    mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -e "UPDATE crib.info_instances SET server_id=$SERVER_ID_NUM, mysql_version='$MYSQL_VERSION_NUM', os='$OS_TYPE', architecture='$OS_ARCH' WHERE hostname='$HOST_NAME' and port=$PORT_NUM"
else
    mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -e "INSERT into crib.info_instances (hostname,port,server_id,mysql_version,os,architecture) values ('$HOST_NAME',$PORT_NUM,$SERVER_ID_NUM,'$MYSQL_VERSION_NUM','$OS_TYPE','$OS_ARCH')"
fi

INSTANCE_ID=`mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT -s -B -e "select id from crib.info_instances WHERE hostname='$HOST_NAME' and port='$PORT_NUM' order by 1 asc limit 1"`

#Avoiding the script overwriting a dump file with a coincidental run of another script 
rand=abc$RANDOM$RANDOM$RANDOM$RANDOM$RANDOM$RANDOM
mkdir /tmp/$rand
chmod 777 /tmp/$rand

mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT crib_client -e "CREATE TABLE $rand as SELECT $INSTANCE_ID, current_timestamp(), table_schema, table_name, engine, ROUND(data_length/1024/1024,0), ROUND(index_length/1024/1024,0), ROUND(data_length/1024/1024,0)+ROUND(index_length/1024/1024,0), ROUND(data_free/1024/1024,0), table_rows FROM information_schema.tables WHERE table_schema != 'information_schema' and table_schema != 'mysql' and table_schema != 'securich' and table_schema != 'crib_client' and table_schema not like '%test%' and engine is not null"

mysqldump -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT --add-drop-table --opt crib_client $rand |  mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib

mysql -u$SRC_USER -p$SRC_PASSWORD -h$SRC_HOSTNAME -P$SRC_PORT crib_client -e "DROP TABLE if exists $rand"

mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib -e "insert into info_object_size select * from $rand"
mysql -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib -e "DROP TABLE if exists $rand"


#mysqlimport -L -u$DST_USER -p$DST_PASSWORD -h$DST_HOSTNAME -P$DST_PORT crib /tmp/$rand/info_object_size.txt

#rm -rf /tmp/$rand
